@@ -7,7 +7,11 @@ from io import StringIO
 from types import SimpleNamespace
 import warnings
 import numpy as np
+
 import matplotlib.pyplot as plt
+from matplotlib.backends.backend_agg import FigureCanvasAgg
+from matplotlib.figure import Figure
+
 from pyyeti import ytools, locate, writer, guitools
 from ._utilities import _get_rpt_headers, _get_numform, _proc_filterval
 from ._magpct import magpct
@@ -292,24 +296,27 @@ def _figure_on(name, doabsmax, show_figures):
     if doabsmax:
         figsize[1] /= 3.0
     if show_figures:
-        plt.figure(name, figsize=figsize)
-        plt.clf()
+        fig = plt.figure(name, figsize=figsize, clear=True)
     else:
-        plt.figure(figsize=figsize)
+        fig = Figure(figsize=figsize)
+        FigureCanvasAgg(fig)
+    return fig
 
 
 def _figure_off(show_figures):
-    if not show_figures:
-        plt.close()
+    pass
+    # if not show_figures:
+    #     plt.close()
 
 
-def _prep_subplot(pctinfo, sp):
+def _prep_subplot(pctinfo, fig, sp):
     if "mx" in pctinfo:
         # if not just doing absmax
         if sp > 311:
-            plt.subplot(sp, sharex=plt.gca())
-        else:
-            plt.subplot(sp)
+            prev_axes = fig.gca()
+            return fig.add_subplot(sp, sharex=prev_axes)
+        return fig.add_subplot(sp)
+    return fig.add_subplot()
 
 
 def _plot_magpct(
@@ -329,14 +336,14 @@ def _plot_magpct(
     ptitle = f"{desc} - {{}} Comparison vs Magnitude"
     xl = f"{names[1]} Magnitude"
     yl = f"% Diff of {names[0]} vs {names[1]}"
-    _figure_on("Magpct - " + desc, doabsmax, show_figures)
+    fig = _figure_on("Magpct - " + desc, doabsmax, show_figures)
     try:
         for lbl, hdr, sp, ismax in (
             ("mx", maxhdr, 311, True),
             ("mn", minhdr, 312, False),
             ("amx", absmhdr, 313, True),
         ):
-            _prep_subplot(pctinfo, sp)
+            axes = _prep_subplot(pctinfo, fig, sp)
             if lbl in pctinfo:
                 if use_range:
                     ref = pctinfo["amx"]["mag"][1]
@@ -348,15 +355,16 @@ def _plot_magpct(
                     Ref=ref,
                     ismax=ismax,
                     filterval=pctinfo[lbl]["magfilt"],
+                    ax=axes,
                     **magpct_options,
                 )
-                plt.title(ptitle.format(hdr))
-                plt.xlabel(xl)
-                plt.ylabel(yl)
-            plt.grid(True)
-        plt.tight_layout(**tight_layout_args)
+                axes.set_title(ptitle.format(hdr))
+                axes.set_xlabel(xl)
+                axes.set_ylabel(yl)
+            axes.grid(True)
+        fig.tight_layout(**tight_layout_args)
         if isinstance(filename, str):
-            plt.savefig(filename + ".magpct.png")
+            fig.savefig(filename + ".magpct.png")
     finally:
         _figure_off(show_figures)
 
@@ -377,14 +385,14 @@ def _plot_histogram(
     ptitle = f"{desc} - {{}} Comparison Histogram"
     xl = f"% Diff of {names[0]} vs {names[1]}"
     yl = "Percent Occurrence (%)"
-    _figure_on("Histogram - " + desc, doabsmax, show_figures)
+    fig = _figure_on("Histogram - " + desc, doabsmax, show_figures)
     try:
         for lbl, hdr, sp in (
             ("mx", maxhdr, 311),
             ("mn", minhdr, 312),
             ("amx", absmhdr, 313),
         ):
-            _prep_subplot(pctinfo, sp)
+            axes = _prep_subplot(pctinfo, fig, sp)
             if lbl in pctinfo:
                 width = histogram_inc
                 x = pctinfo[lbl]["hsto"][:, 0]
@@ -396,17 +404,17 @@ def _plot_histogram(
                 for pv, c in ((pv1, "m"), (pv2, "r")):
                     for i in pv:
                         colors[i] = c
-                plt.bar(x, y, width=width, color=colors, align="center")
-                plt.title(ptitle.format(hdr))
-                plt.xlabel(xl)
-                plt.ylabel(yl)
-                x = abs(max(plt.xlim(), key=abs))
+                axes.bar(x, y, width=width, color=colors, align="center")
+                axes.set_title(ptitle.format(hdr))
+                axes.set_xlabel(xl)
+                axes.set_ylabel(yl)
+                x = abs(max(axes.get_xlim(), key=abs))
                 if x < 5:
-                    plt.xlim(-5, 5)
-            plt.grid(True)
-        plt.tight_layout(**tight_layout_args)
+                    axes.set_xlim(-5, 5)
+            axes.grid(True)
+        fig.tight_layout(**tight_layout_args)
         if isinstance(filename, str):
-            plt.savefig(filename + ".histogram.png")
+            fig.savefig(filename + ".histogram.png")
     finally:
         _figure_off(show_figures)
 
@@ -594,35 +602,28 @@ def rptpct1(
     magpct_options : None or dict; must be named; optional
         If None, it is internally reset to::
 
-            magpct_options = {'filterval': 'filterval'}
+            magpct_options = {'filterval': 'same'}
 
         Use this parameter to provide any options to :func:`magpct`
         but note that the `filterval` option for :func:`magpct` is
         treated specially. Here, in addition to any of the values that
         :func:`magpct` accepts, it can also be set to the string
-        "filterval" as in the default case shown above. In that case,
+        "same" as in the default case shown above. If set to "same",
         ``magpct_options['filterval']`` gets internally reset to the
-        initial value of `filterval` (which is None by default).
+        final value of `filterval` so that the comparison table, the
+        histogram, and the magpct plot all use the same filter value.
+        For backward compatibility, the string "filterval" is accepted
+        as well and works like "same".
 
         .. note::
             The call to :func:`magpct` is *after* applying `ignorepv`
             and doing any data aligning by labels.
 
         .. note::
-           The two filter value options (`filterval` and
-           ``magpct_options['filterval']``) have different defaults:
-           None and 'filterval`, respectively. They also differ on how
-           the ``None`` setting is used: for `filterval`, None is
-           replaced by 1.e-6 while for `magpct_filterval`, None means
-           that the "magpct" plot will not have any filters applied at
-           all.
-
-        .. note::
-            The above means that, if you accept the default values for
-            `filterval` and for ``magpct_options['filterval']``, then
-            tables and the histogram plots will use a `filterval` of
-            1.e-6 while the "magpct" plots will use no filter (it
-            compares everything except perfect zeros).
+           Unless the :func:`magpct` option `plot_all` is set to
+           False, all values (even those smaller than `filterval`) are
+           compared and shown on the :func:`magpct` plot in the shaded
+           region.
 
     doabsmax : bool; must be named; optional
         If True, compare only absolute maximums.
@@ -798,23 +799,25 @@ def rptpct1(
         tight_layout_args = {"pad": 3.0}
 
     if magpct_options is None:
-        magpct_options = {"filterval": "filterval"}
+        magpct_options = {"filterval": "same"}
     else:
         magpct_options = magpct_options.copy()
 
-    # magpct_options['filterval'] get special treatment:
+    # magpct_options['filterval'] gets special treatment:
     magpct_filterval = magpct_options["filterval"]
     del magpct_options["filterval"]
 
+    reset_magpct_filterval = False
     if isinstance(magpct_filterval, str):
-        if magpct_filterval != "filterval":
+        if magpct_filterval in ("same", "filterval"):
+            reset_magpct_filterval = True
+            magpct_filterval = None
+        else:
             raise ValueError(
                 "``magpct_options['filterval']`` is an invalid "
                 f"string: {magpct_filterval!r} (can only "
                 "be 'filterval' if a string)"
             )
-        # copy the initial `filterval` setting:
-        magpct_filterval = filterval
 
     infovars = (
         "desc",
@@ -877,9 +880,12 @@ def rptpct1(
     if filterval is None:
         filterval = 1.0e-6
     filterval = _proc_filterval(filterval, R, "filterval")
-    magpct_filterval = _proc_filterval(
-        magpct_filterval, R, "magpct_options['filterval']"
-    )
+    if reset_magpct_filterval:
+        magpct_filterval = filterval
+    else:
+        magpct_filterval = _proc_filterval(
+            magpct_filterval, R, "magpct_options['filterval']"
+        )
 
     if labels is None:
         labels = [f"Row {i + 1:6d}" for i in range(R)]
