@@ -693,11 +693,11 @@ class DR_Def(OrderedDict):
 
         **Entering partition vectors.** The `histpv`, `srspv` and
         `ignorepv` inputs are all handled similarly. They can be input
-        as either an index or boolean style partition vector. If input
-        as 'all', they are reset internally to ``slice(len(labels))``.
-        They can also be entered as a slice directly. The stored
-        versions are either a slice or an index vector (uses 0-offset
-        for standard Python indexing).
+        as an index or boolean style partition vector, as a slice, or
+        as "all". If input as 'all', they are reset internally to
+        ``slice(len(labels))``. The stored versions are either a slice
+        or an index vector (uses 0-offset for standard Python
+        indexing).
 
         **`drfunc`, `drfile` notes.** If `drfunc` is a valid Python
         identifier (called "Type 2" above), then `drfile` must contain
@@ -1036,7 +1036,8 @@ class DR_Def(OrderedDict):
         def _issame(old, new):
             if new is old:
                 return True
-            if type(new) != type(old):
+            # if type(new) != type(old):
+            if not isinstance(new, type(old)):
                 return False
             if isinstance(new, np.ndarray):
                 if new.shape != old.shape:
@@ -1066,7 +1067,8 @@ class DR_Def(OrderedDict):
                             pass
                         else:
                             s = f"{slen}: {s}"
-                df[cat].loc[val] = s
+                # df[cat].loc[val] = s
+                df.loc[val, cat] = s
 
         if excel_file is not None:
             with xlsxwriter.Workbook(excel_file) as workbook:
