@@ -4,11 +4,11 @@ DR_Event: Setup data recovery for a specific event or set of modes
 """
 import copy
 from types import SimpleNamespace
-import warnings
 from collections import OrderedDict
 import numpy as np
 import scipy.linalg as la
 from pyyeti.ytools import reorder_dict
+from pyyeti.locate import flippv, index2slice
 from pyyeti.nastran import n2p
 from .dr_results import DR_Results
 from ._utilities import _merge_uf_reds
@@ -434,9 +434,10 @@ class DR_Event:
             Modal stiffness; vector or matrix
         nrb : scalar
             Number of rigid-body modes
-        rfmodes : index vector or None
-            Specifies where the res-flex modes are; if None, no
-            resflex
+        rfmodes : 1d array or None
+            Index or bool partition vector specifying where the
+            residual-flexibility modes are; if None, there are no
+            res-flex vectors.
 
         Returns
         -------
@@ -485,118 +486,10 @@ class DR_Event:
 
               F = m*a + b*v + k*d
         """
-        n = k.shape[0]
-        use_velo = True
-
-        # genforce = m*a + b*v + k*d
-        def _comp_genforce(sol, d):
-            if m is None:
-                genforce = sol.a.copy()
-            elif m.ndim == 1:
-                genforce = m[:, None] * sol.a
-            else:
-                genforce = m @ sol.a
-
-            if b.ndim == 1:
-                genforce += b[:, None] * sol.v
-            else:
-                genforce += b @ sol.v
-
-            if k.ndim == 1:
-                genforce += k[:, None] * d
-            else:
-                genforce += k @ d
-            return genforce
-
         solout = {}
-        # This routine will do its work in two loops:
-        # - a copy loop
-        # - an apply ufs loop
-        # The copy loop is separate to be efficient, since copying
-        # from 'F' order to 'C' order is slower than 'C' to 'C'
-        last = None
-        for item in self.UF_reds:
-            if last:
-                solout[item] = copy.deepcopy(last)
-            else:
-                # we want C contiguous a, v, d variables for faster uf
-                # application
-                # - variable 'd' is skipped because it is created
-                #   below
-                solout[item] = SimpleNamespace()
-                for name, value in vars(sol).items():
-                    if name == "d":
-                        continue
-                    try:
-                        valcopy = value.copy()
-                    except AttributeError:
-                        valcopy = copy.deepcopy(value)
-                    setattr(solout[item], name, valcopy)
-                last = solout[item]
-                genforce = _comp_genforce(last, sol.d)
-
-        for item in self.UF_reds:
-            ruf, euf, duf, suf = item
-            SOL = solout[item]
-            SOL.d_static = np.empty_like(SOL.a)
-            SOL.d_dynamic = np.empty_like(SOL.a)
-
-            # apply ufs:
-            if nrb > 0:
-                SOL.a[:nrb] *= ruf * suf
-                SOL.v[:nrb] *= ruf * suf
-                SOL.d_static[:nrb] = 0.0
-                SOL.d_dynamic[:nrb] = 0.0
-
-            if rfmodes is not None:
-                SOL.a[rfmodes] = 0.0
-                SOL.v[rfmodes] = 0.0
-
-            try:
-                SOL.pg *= suf
-            except AttributeError:
-                pass
-
-            if nrb < n:
-                SOL.a[nrb:] *= euf * duf
-                SOL.v[nrb:] *= euf * duf
-
-                if m is None:
-                    avterm = SOL.a[nrb:].copy()
-                elif m.ndim == 1:
-                    avterm = m[nrb:, None] * SOL.a[nrb:]
-                else:
-                    avterm = m[nrb:, nrb:] @ SOL.a[nrb:]
-
-                if not use_velo:  # pragma: no cover
-                    msg = (
-                        "not including velocity term in mode-"
-                        "acceleration formulation for "
-                        "displacements."
-                    )
-                    warnings.warn(msg, RuntimeWarning)
-                else:
-                    if b.ndim == 1:
-                        avterm += b[nrb:, None] * SOL.v[nrb:]
-                    else:
-                        avterm += b[nrb:, nrb:] @ SOL.v[nrb:]
-
-                # in case there is mass coupling between rfmodes and
-                # other modes
-                if rfmodes is not None:
-                    avterm[rfmodes - nrb] = 0.0
-
-                gf = (euf * suf) * genforce[nrb:]
-                if k.ndim == 1:
-                    invk = (1 / k[nrb:])[:, None]
-                    SOL.d_static[nrb:] = invk * gf
-                    SOL.d_dynamic[nrb:, :] = -invk * avterm
-                else:
-                    lup = la.lu_factor(k[nrb:, nrb:])
-                    SOL.d_static[nrb:] = la.lu_solve(lup, gf)
-                    SOL.d_dynamic[nrb:, :] = la.lu_solve(lup, -avterm)
-
-                SOL.d = SOL.d_static + SOL.d_dynamic
+        save = {}
+        for uf_reds in self.UF_reds:
+            solout[uf_reds] = apply_uf(sol, uf_reds, m, b, k, nrb, rfmodes, save)
         return solout
 
     def frf_apply_uf(self, sol, nrb):
@@ -673,3 +566,215 @@ class DR_Event:
             if v.srsQs is not None:
                 Qs |= set(v.srsQs)  # or: Qs = Qs.union(v.srsQs)
         return sorted(Qs)
+
+
+# genforce = non-rb part of: m*a + b*v + k*d
+# avterm = non-rb, non-rf part of: m*a + b*v
+# - rfmodes is an index partition vector
+def _pre_calcs(sol, m, b, k, nrb, rfmodes, save):
+    n = sol.a.shape[0]
+    genforce = np.empty((n - nrb, sol.a.shape[1]), sol.a.dtype)
+
+    if rfmodes is not None:
+        elastic = flippv(rfmodes, n)[nrb:]
+        elastic_norb = index2slice(elastic - nrb)
+        elastic = index2slice(elastic)
+        rf_norb = rfmodes - nrb
+    else:
+        elastic = slice(nrb, None)
+        elastic_norb = slice(n - nrb)
+        rf_norb = None
+
+    if isinstance(elastic, slice):
+        ee = (elastic, elastic)
+    else:
+        ee = np.ix_(elastic, elastic)
+
+    if m is None:
+        genforce[elastic_norb] = sol.a[elastic]
+    elif m.ndim == 1:
+        genforce[elastic_norb] = m[elastic, None] * sol.a[elastic]
+    else:
+        genforce[elastic_norb] = m[ee] @ sol.a[elastic]
+
+    if b.ndim == 1:
+        genforce[elastic_norb] += b[elastic, None] * sol.v[elastic]
+    else:
+        genforce[elastic_norb] += b[ee] @ sol.v[elastic]
+
+    avterm = genforce[elastic_norb]
+    if avterm.base is not None:
+        # ensure copy, not view:
+        avterm = avterm.copy()
+
+    if k.ndim == 1:
+        genforce[elastic_norb] += k[elastic, None] * sol.d[elastic]
+        if rfmodes is not None:
+            genforce[rf_norb] = k[rfmodes, None] * sol.d[rfmodes]
+    else:
+        kee = k[ee]
+        genforce[elastic_norb] += kee @ sol.d[elastic]
+        if kee.shape[0] > 0:
+            save["lup_elastic"] = la.lu_factor(kee, check_finite=False)
+        else:
+            save["lup_elastic"] = None
+        save["lup_rf"] = None
+        if rfmodes is not None:
+            krr = k[np.ix_(rfmodes, rfmodes)]
+            genforce[rf_norb] = krr @ sol.d[rfmodes]
+            if krr.shape[0] > 0:
+                save["lup_rf"] = la.lu_factor(krr, overwrite_a=True, check_finite=False)
+
+    save["genforce"] = genforce
+    save["avterm"] = avterm
+    save["elastic"] = elastic
+    save["elastic_norb"] = elastic_norb
+    save["rf_norb"] = rf_norb
+
+
+def apply_uf(sol, uf_reds, m, b, k, nrb, rfmodes, save=None):
+    """
+    Applies the uncertainty factors to the modal ODE solution
+
+    Parameters
+    ----------
+    sol : SimpleNamespace
+        Solution, input only; expected to have::
+
+            .a = modal acceleration time-history matrix
+            .v = modal velocity time-history matrix
+            .d = modal displacement time-history matrix
+            .pg = g-set forces; optional
+
+    uf_reds : 4-element tuple
+        This is the uncertainty factors in "reds" order: [rigid,
+        elastic, dynamic, static].
+    m : 1d or 2d ndarray or None
+        Modal mass; can be vector or matrix or None (for identity)
+    b : 1d or 2d ndarray
+        Modal damping; vector or matrix
+    k : 1d or 2d ndarray
+        Modal stiffness; vector or matrix
+    nrb : scalar
+        Number of rigid-body modes
+    rfmodes : 1d array or None
+        Index or bool partition vector specifying where the
+        residual-flexibility modes are; if None, there are no
+        res-flex vectors.
+    save : None or dict; optional
+        For efficiency. When calling this routine multiple times to
+        apply different uncertainty factors (that is, only `uf_reds`
+        is changing), set `save` to an empty dict; this routine will
+        put items in `save` for subsequent calls.
+
+    Returns
+    -------
+    solout : SimpleNamespace
+
+        The scaled version of `sol`. Additionally, the displacement member is
+        separated into static and dynamic parts::
+
+            .a
+            .v
+            .d
+            .d_static
+            .d_dynamic
+            .pg (optional)
+
+        where::
+
+            solout.d = solout.d_static + solout.d_dynamic
+
+    Notes
+    -----
+    Uncertainty factors are applied as follows (rb=rigid-body,
+    el=elastic, rf=residual-flexibility)::
+
+       ruf = rb uncertainty factor
+       euf = el uncertainty factor
+       duf = dynamic uncertainty factor
+       suf = static uncertainty factor
+
+       .a_rb and .v_rb - scaled by ruf*suf
+       .a_el and .v_el - scaled by euf*duf
+       .a_rf and .v_rf - zeroed out
+
+       .d_rb - zeroed out
+       .d_el - static part:  scaled by euf*suf
+             - dynamic part: scaled by euf*duf
+       .d_rf - scaled by euf*suf
+
+       .pg   - scaled by suf
+
+    Note that d_el is written out as::
+
+          d_el = euf*inv(k_el)*(suf*F_el - duf*(a_el+b_el*v_el))
+
+    where::
+
+          F = m*a + b*v + k*d
+    """
+    # ensure that rfmodes is an index type:
+    if rfmodes is not None:
+        rfmodes = np.atleast_1d(rfmodes)
+        if np.issubdtype(rfmodes.dtype, np.bool_):
+            rfmodes = rfmodes.nonzero()[0]
+
+    ruf, euf, duf, suf = uf_reds
+
+    solout = SimpleNamespace(**vars(sol))
+    # don't change original a, v, d, or pg (d is created below)
+    solout.a = solout.a.copy()
+    solout.v = solout.v.copy()
+    try:
+        solout.pg = sol.pg * suf
+    except AttributeError:
+        pass
+
+    solout.d_static = np.empty_like(solout.a)
+    solout.d_dynamic = np.empty_like(solout.a)
+
+    # apply ufs:
+    if nrb > 0:
+        solout.a[:nrb] *= ruf * suf
+        solout.v[:nrb] *= ruf * suf
+        solout.d_static[:nrb] = 0.0
+        solout.d_dynamic[:nrb] = 0.0
+
+    if nrb == k.shape[0]:
+        solout.d = solout.d_static + solout.d_dynamic
+        return solout
+
+    if rfmodes is not None:
+        solout.a[rfmodes] = 0.0
+        solout.v[rfmodes] = 0.0
+        solout.d_dynamic[rfmodes] = 0.0
+
+    if save is None:
+        save = {}
+
+    if "genforce" not in save:
+        _pre_calcs(sol, m, b, k, nrb, rfmodes, save)
+
+    avterm = (euf * duf) * save["avterm"]
+    solout.a[nrb:] *= euf * duf
+    solout.v[nrb:] *= euf * duf
+
+    gf = (euf * suf) * save["genforce"]
+    elastic = save["elastic"]
+    elastic_norb = save["elastic_norb"]
+    if k.ndim == 1:
+        kel = k[nrb:, None]
+        solout.d_static[nrb:] = gf / kel
+        solout.d_dynamic[elastic] = -avterm / kel[elastic_norb]
+        solout.d = solout.d_static + solout.d_dynamic
+        return solout
+
+    if (lup := save["lup_elastic"]) is not None:
+        solout.d_static[elastic] = la.lu_solve(lup, gf[elastic_norb])
+        solout.d_dynamic[elastic] = la.lu_solve(lup, -avterm)
+    if (lup := save["lup_rf"]) is not None:
+        solout.d_static[rfmodes] = la.lu_solve(lup, gf[save["rf_norb"]])
+
+    solout.d = solout.d_static + solout.d_dynamic
+    return solout
