@@ -20,12 +20,15 @@ else:
 try:
     import pyyeti.rainflow.c_rain as rain
 except ImportError:
-    warnings.warn(
-        "Compiled C version of rainflow algorithm failed to "
-        "import. Using plain Python version, which can be very "
-        "slow if 'numba' is not installed.",
-        RuntimeWarning,
-    )
+    if not HAVE_NUMBA:
+        warnings.warn(
+            "Compiled C version of rainflow algorithm failed to "
+            "import. Using plain Python version, which can be very "
+            "slow. Recommend installing 'numba': pyYeti will use that "
+            "automatically to speed up the Python version to be on "
+            "par with the C version.",
+            RuntimeWarning,
+        )
     import pyyeti.rainflow.py_rain as rain
 
 # FIXME: We need the str/repr formatting used in Numpy < 1.14.
@@ -42,8 +45,8 @@ def rainflow(peaks, getoffsets=False, use_pandas=True):
     Parameters
     ----------
     peaks : 1d array_like
-        Vector of alternating peaks (as returned by
-        :func:`pyyeti.findap`, for example)
+        Vector of alternating peaks (as returned by :func:`findap`,
+        for example)
     getoffsets : bool; optional
         If True, the tuple ``(rf, os)`` is returned; otherwise, only
         `rf` is returned.
@@ -53,7 +56,6 @@ def rainflow(peaks, getoffsets=False, use_pandas=True):
     Returns
     -------
     rf : pandas DataFrame or 2d ndarray
-
         n x 3 matrix with the rainflow cycle count information with
         the index going from 0 to n-1 and the columns being ['amp',
         'mean', 'count']:
@@ -64,7 +66,6 @@ def rainflow(peaks, getoffsets=False, use_pandas=True):
               half or full cycle
 
     os : pandas DataFrame or 2d ndarray
-
         Only returned if `getoffsets` is True. n x 2 matrix of cycle
         offsets with index going from 0 to n-1 and the columns being
         ['start', 'stop']:
@@ -75,9 +76,13 @@ def rainflow(peaks, getoffsets=False, use_pandas=True):
     Notes
     -----
     This algorithm is derived from reference [#cc1]_. This routine is
-    a wrapper for either the :func:`pyyeti.rainflow.c_rain` or
-    :func:`pyyeti.rainflow.py_rain` routines. Note that the C version
-    is *much* faster (the algorithms are the same).
+    a wrapper for either the C version
+    (:func:`pyyeti.rainflow.c_rain`) or the Python version
+    (:func:`pyyeti.rainflow.py_rain`) of the algorithm. Both versions
+    give identical results. They also run in comparable times if
+    :func:`numba.jit` is available (if ``import numba`` works). If
+    :func:`numba.jit` is not available, the C version is *much*
+    faster.
 
     References
     ----------
@@ -244,7 +249,6 @@ if not HAVE_NUMBA:
         # [ True, False, False,  True, False,  True, False, False]
         return PV
 
-
 else:
 
     def findap(y, tol=1e-6):
@@ -382,6 +386,11 @@ def getbins(bins, mx, mn, right=True, check_bounds=False):
 
     Notes
     -----
+    If `mx` and `mn` are equal, they are reset::
+
+        mx = mx + 0.5
+        mn = mn - 0.5
+
     If `bins` is a scalar, this routine tries to mimic the behavior of
     the :func:`pandas.cut` routine::
 
@@ -399,8 +408,6 @@ def getbins(bins, mx, mn, right=True, check_bounds=False):
 
     Raises
     ------
-    ValueError
-        If `mx` and `mn` are equal
     ValueError
         If `bins` is a vector but is not monotonically increasing
 
@@ -418,9 +425,12 @@ def getbins(bins, mx, mn, right=True, check_bounds=False):
     if mx < mn:
         mx, mn = mn, mx
     elif mx == mn:
-        raise ValueError("`mx` and `mn` must not be equal")
+        mx = mx + 0.5
+        mn = mn - 0.5
+        # raise ValueError("`mx` and `mn` must not be equal")
+
     if bins.size == 1:
-        bins = int(bins)
+        bins = int(bins[0])
         bb = np.linspace(mn, mx, bins + 1)
         p = 0.001 * (mx - mn)
         if right:
@@ -495,7 +505,6 @@ if HAVE_NUMBA:
                 mn = v
         return mx, mn
 
-
 else:
 
     def maxmin(x):
@@ -598,13 +607,13 @@ def binify(
     6  3.0   1.0    0.5
     >>> format = lambda x: f"{x:.1f}"
     >>> df = cyclecount.binify(rf, 3, 2)
-    >>> df.applymap(format)                # doctest: +ELLIPSIS
+    >>> df.map(format)                # doctest: +ELLIPSIS
     Amp             (1.497, 2.500] (2.500, 3.500] (3.500, 4.500]
     Mean...
     (-1.002, 0.000]            1.0            0.0            0.5
     (0.000, 1.000]             1.0            0.5            1.0
     >>> df = cyclecount.binify(rf, 3, 2, right=0)
-    >>> df.applymap(format)                # doctest: +ELLIPSIS
+    >>> df.map(format)                # doctest: +ELLIPSIS
     Amp             [1.500, 2.500) [2.500, 3.500) [3.500, 4.503)
     Mean...
     [-1.000, 0.000)            1.0            0.0            0.0
@@ -704,9 +713,9 @@ def sigcount(
     Notes
     -----
     Steps:
-      1.  calls :func:`findap` to find all local minima and maxima:
+      1.  Calls :func:`findap` to find all local minima and maxima:
           ``peaks = sig[findap(sig)]``
-      2.  calls :func:`rainflow` to do the cycle counting:
+      2.  Calls :func:`rainflow` to do the cycle counting:
           ``rf = rainflow(peaks)``
       3.  Summarizes the rainflow counting by putting the counts in
           bins. For each mean value bin (i'th):
