@@ -18,6 +18,9 @@ from matplotlib.lines import Line2D
 from mpl_toolkits.mplot3d import proj3d
 
 
+from time import sleep
+
+
 def mk_label(point):
     """Default annotation function"""
 
@@ -26,11 +29,17 @@ def mk_label(point):
             return f"{xyz:.5g}"
         return f"{xyz}"
 
-    label = f"x: {tostr(point.x)}\ny: {tostr(point.y)}"
-    if point.z is not None:
-        label += f"\nz: {tostr(point.z)}"
+    if point.ax.name == "polar":
+        deg = np.rad2deg(point.x % (2 * np.pi))
+        label = f"θ: {tostr(point.x)} ({tostr(deg)}°)\nrad: {tostr(point.y)}"
+    else:
+        label = f"x: {tostr(point.x)}\ny: {tostr(point.y)}"
+        if point.z is not None:
+            label += f"\nz: {tostr(point.z)}"
+
     if point.nlines > 1:
         label += f"\n{point.handle.get_label()}"
+
     return label
 
 
@@ -72,26 +81,43 @@ class DataCursor(object):
                         return f"{xyz:.5g}"
                     return f"{xyz}"
 
-                label = f"x: {tostr(point.x)}\ny: {tostr(point.y)}"
-                if point.z is not None:
-                    label += f"\nz: {tostr(point.z)}"
+                if point.ax.name == "polar":
+                    deg = np.rad2deg(point.x % (2 * np.pi))
+                    label = (
+                        f"θ: {tostr(point.x)} ({tostr(deg)}°)\n"
+                        f"rad: {tostr(point.y)}"
+                    )
+                else:
+                    label = f"x: {tostr(point.x)}\ny: {tostr(point.y)}"
+                    if point.z is not None:
+                        label += f"\nz: {tostr(point.z)}"
+
                 if point.nlines > 1:
                     label += f"\n{point.handle.get_label()}"
+
                 return label
 
         The SimpleNamespace ``point`` contains the following
         attributes:
 
-        ====== ========================================================
-         attr  Description
-        ====== ========================================================
-        ax     Axes handle
-        handle Line2D or PathCollection handle for line
-        index  Index into the data vectors for data point
-        lines  Total number of lines on axes
-        n      Line number starting at 0
-        x,y,z  x, y, z coordinates of data point; z will be None for 2D
-        ====== ========================================================
+        ======= ======================================================
+         attr   Description
+        ======= ======================================================
+        ax      Axes handle
+        handle  Line2D or PathCollection handle for line
+        index   Index into the data vectors for data point
+        lines   Total number of lines on axes
+        n       Line number starting at 0
+        x,y,z   x, y, z coordinates of data point; z is None for 2D
+        dot     The :func:`matplotlib.pyplot.scatter` (PathCollection)
+                object handle. Note that :class:`DataCursor` ignores
+                these added annotation points when moused over; they
+                are identified by the added attribute
+                "_pyyeti_dc_point".
+        note    The annotation object handle from
+                :func:`matplotlib.pyplot.annotate`.
+        xy_note The (x, y) coordinates for the note.
+        ======= ======================================================
 
     offsets : tuple
         Two element tuple containing x and y offsets in points for the
@@ -114,16 +140,13 @@ class DataCursor(object):
     points : list
         Contains list of SimpleNamespace objects. Each "point" is as
         described above under the `mk_label` attribute.
-    pts : list
-        Contains the :func:`matplotlib.pyplot.scatter`
-        (PathCollection) object handles for the selected and annotated
-        points. Same length as `points`. Note that
-        :class:`DataCursor` ignores these added annotation points when
-        moused over; they are identified by the added attribute
-        "_pyyeti_dc_point".
-    notes : list
-        Contains the annotation object handles for the selected points.
-        Same length as `points`.
+    xyz : 2d ndarray or None
+        If points have been selected and the datacursor has been
+        turned off, the `xyz` attribute will be filled with the x, y,
+        z data for the selected points for all axes. If there are no
+        3d plots, only the x and y columns will be present. The order
+        of the rows follows the selection order (same order as
+        `points`; the source of this data).
 
     Notes
     -----
@@ -148,21 +171,18 @@ class DataCursor(object):
     to see selected data points. These operations are available (when
     the mouse is inside the axes):
 
-    ===========  =====================================================
+    ===========  ======================================================
     Action       Description
-    ===========  =====================================================
+    ===========  ======================================================
     left-click   Data point will be stored in the member list
-                 `points`. The `pts` and `notes` attributes are
-                 updated as well.
-    right-click  Last point is deleted from `points` and from the
-                 other members.
+                 `points`.
+    right-click  Last point is deleted from the plot and from `points`.
     typing 't'   Turns off DataCursor. To turn on, use ``DC.on``. Note
-                 that turning on will reset `points` and the other
-                 members.
+                 that turning on will reset `points`.
     typing 'D'   Deletes last point AND removes the line from the plot
                  via ``line_handle.remove()``. Any older annotations
                  are not deleted.
-    ===========  =====================================================
+    ===========  ======================================================
 
     To get data points from plots from within a script, use
     :func:`DataCursor.getdata`. Enter the number of points or press
@@ -189,8 +209,9 @@ class DataCursor(object):
         import matplotlib.pyplot as plt
         import numpy as np
         from pyyeti.datacursor import DC
+        rng = np.random.default_rng()
         x = np.arange(500)/250
-        y = np.random.rand(*x.shape)
+        y = rng.uniform(size=x.shape)
         fig = plt.figure('demo')
         fig.clf()
         ax = fig.add_subplot(1, 1, 1)
@@ -219,8 +240,9 @@ class DataCursor(object):
         DC.permdot = dict(s=130, color='black', alpha=0.4,
                           marker='p')
 
-        plt.plot(np.random.randn(50), label='Gaussian')
-        plt.plot(np.random.rand(50), label='Uniform')
+        rng = np.random.default_rng()
+        plt.plot(rng.normal(size=50), label='Gaussian')
+        plt.plot(rng.uniform(size=50), label='Uniform')
         DC.on()
 
     For increased versatility, there are two optional functions the
@@ -245,7 +267,8 @@ class DataCursor(object):
 
         fig = plt.figure()
         ax = fig.add_subplot(projection="3d")
-        coords = np.random.randn(6, 3)
+        rng = np.random.default_rng()
+        coords = rng.normal(size=(6, 3))
         dots = ax.scatter(*coords.T)
         ax.plot(*(coords.T + 0.1), "v")
         DC.on()
@@ -380,12 +403,12 @@ class DataCursor(object):
         self._init_all()
         if reset:
             self.points = []
-            self.pts = []
-            self.notes = []
+            self.xyz = None
         self._kid = {}  # key press event
         self._mid = {}  # motion event
         self._bid = {}  # button press event
         self._aid = {}  # axes leave event
+        self._fig_layout_engine = {}
         self._setup_annotations()
         if callbacks:
             for fig in self._figs:
@@ -396,6 +419,9 @@ class DataCursor(object):
                         self._mid[fig] = cvs.mpl_connect(
                             "motion_notify_event", self._follow
                         )
+                        if hasattr(fig, "get_layout_engine"):
+                            self._fig_layout_engine[fig] = fig.get_layout_engine()
+                            fig.set_layout_engine("none")
                     else:
                         self._mid[fig] = None
                     self._bid[fig] = cvs.mpl_connect("button_press_event", self._follow)
@@ -428,18 +454,21 @@ class DataCursor(object):
             for ax in self._ax:
                 if ax in self._annotation:
                     self._annotation[ax].set_visible(False)
-                    self._dot[ax].set_visible(False)
+                    self._mk_followdot_invisible(ax)
             for fig in self._figs:
                 if fig in self._kid:
                     fig.canvas.mpl_disconnect(self._kid[fig])
-                if self.hover and fig in self._mid and self._mid[fig] is not None:
-                    fig.canvas.mpl_disconnect(self._mid[fig])
+                if self.hover:
+                    if fig in self._mid and self._mid[fig] is not None:
+                        fig.canvas.mpl_disconnect(self._mid[fig])
+                    if fig in self._fig_layout_engine:
+                        fig.set_layout_engine(self._fig_layout_engine[fig])
                 if fig in self._bid:
                     fig.canvas.mpl_disconnect(self._bid[fig])
                     fig.canvas.mpl_disconnect(self._aid[fig])
             # make annotations draggable:
-            for an in self.notes:
-                an.draggable()
+            for pt in self.points:
+                pt.note.draggable()
             self._is_on = False
         for fig in self._figs:
             fig.canvas.draw()
@@ -447,6 +476,13 @@ class DataCursor(object):
             self._figs[0].canvas.stop_event_loop()
             self._in_loop = False
 
+        # make xyz:
+        if any(p.z for p in self.points):
+            xyz = [[p.x, p.y, p.z if p.z is not None else np.nan] for p in self.points]
+        else:
+            xyz = [[p.x, p.y] for p in self.points]
+        self.xyz = np.array(xyz)
+
     def addpt_func(self, func):
         """
         Function to call on a left-click.
@@ -471,21 +507,39 @@ class DataCursor(object):
 
     @staticmethod
     def _add_annotation_point(ax, x, y, dct, vis):
+        # get/set axis limits; this is strange, but ax.scatter will
+        # rescale the plots for some reason in some cases
+        for lim in ("xlim", "ylim", "zlim"):
+            if fnc := getattr(ax, f"get_{lim}", None):
+                getattr(ax, f"set_{lim}")(fnc())
         h = ax.scatter(x, y, **dct, visible=vis)
         h._pyyeti_dc_point = True
         return h
 
+    def _mk_followdot_invisible(self, ax):
+        if self._dot[ax] is not None:
+            self._dot[ax].set_visible(False)
+
+    def _get_followdot_handle(self, ax, x, y):
+        if self._dot[ax] is None:
+            self._dot[ax] = self._add_annotation_point(ax, x, y, self.followdot, False)
+        else:
+            self._dot[ax].set_offsets((x, y))
+        return self._dot[ax]
+
     @staticmethod
     def _is3d(ax):
         return hasattr(ax, "get_proj")
 
-    def _add_point(self, x, y, point):
-        self.points.append(point)
+    def _add_point(self, point):
         # if 3d, set visible to False (i don't know how to get proper
         # coordinates for the dot):
         vis = False if self._is3d(point.ax) else True
-        self.pts.append(self._add_annotation_point(point.ax, x, y, self.permdot, vis))
-        self.notes.append(self._annotation[point.ax])
+        x, y = point.xy_note
+        point.dot = self._add_annotation_point(point.ax, x, y, self.permdot, vis)
+        point.note = self._annotation[point.ax]
+        self.points.append(point)
+
         # make a new annotation box so current one is static
         self._annotation[point.ax] = self._new_annotation(point.ax, (x, y))
         if self._in_loop and len(self.points) == self._max_points:
@@ -497,19 +551,25 @@ class DataCursor(object):
         """Deletes last saved point, if any."""
         if len(self.points) == 0:
             return
-        point = self.points.pop()
+
+        if ax:
+            for i in reversed(range(len(self.points))):
+                if self.points[i].ax == ax:
+                    point = self.points.pop(i)
+                    break
+            else:
+                return
+
         if delete_line:
             # line may have been deleted already, so catch exception:
+            line = point.handle
             try:
-                h.remove()
+                line.remove()
             except ValueError:
                 pass
-            else:
-                h = None
-        pt = self.pts.pop()
-        pt.remove()
-        na = self.notes.pop()
-        na.remove()
+
+        point.dot.remove()
+        point.note.remove()
         if self.delptfunc:
             self.delptfunc(point)
 
@@ -521,19 +581,7 @@ class DataCursor(object):
         if event.inaxes is None:
             return
         if event.inaxes in self._ax:
-            ax = event.inaxes
-        else:
-            # for overlapping axes, only top axes gets the event, so
-            # check others manually
-            for ax in self._ax:
-                pt = (event.x, event.y)
-                if ax.contains_point(pt):
-                    inv = ax.transData.inverted()
-                    event.xdata, event.ydata = inv.transform_point(pt)
-                    break
-            else:
-                return
-        return ax
+            return event.inaxes
 
     def _key(self, event):
         """
@@ -556,7 +604,7 @@ class DataCursor(object):
         if not ax:
             return
         self._annotation[ax].set_visible(False)
-        self._dot[ax].set_visible(False)
+        self._mk_followdot_invisible(ax)
         event.canvas.draw()
 
     def _follow(self, event):
@@ -565,18 +613,18 @@ class DataCursor(object):
         ax = self._get_ax(event)
         if not ax:
             return
-        x, y, point = self._snap(ax, event.xdata, event.ydata)
-        if x is None:
+        point = self._snap(ax, event.x, event.y)
+        if point is None:
             return
+        x, y = point.xy_note
         annotation = self._annotation[ax]
-        dot = self._dot[ax]
         annotation.xy = x, y
         annotation.set_text(self.mk_label(point))
-        dot.set_offsets((x, y))
+        dot = self._get_followdot_handle(ax, x, y)
         if event.name == "button_press_event":
             if event.button == 1:
                 annotation.set_visible(True)
-                self._add_point(x, y, point)
+                self._add_point(point)
             elif event.button == 3 and len(self.points) > 0:
                 self._del_point(ax=ax)
         elif self.hover:
@@ -618,8 +666,24 @@ class DataCursor(object):
             x, y = (*h.get_offsets().data.T,)
         return x, y
 
+    def _get_xy_data_display(self, ax, h):
+        if isinstance(h, Line2D):
+            x_ann, y_ann = h.get_xdata(), h.get_ydata()
+            x = ax.convert_xunits(x_ann)  # eg, convert dates to floats
+            y = ax.convert_yunits(y_ann)
+        elif hasattr(h, "_offsets3d"):
+            x, y, _ = proj3d.proj_transform(*np.array(h._offsets3d), ax.get_proj())
+            x_ann, y_ann = x, y
+        else:
+            x, y = (*h.get_offsets().data.T,)
+            x_ann, y_ann = x, y
+
+        # get pixel coordinates:
+        x, y = ax.transData.transform(np.column_stack((x, y))).T
+        return x, y, x_ann, y_ann
+
     @staticmethod
-    def _get_xyz(ax, h, ind):
+    def _get_xyz(h, ind):
         """
         Get original 3D location of a point, instead of the mapped-to-2D
         x, y location
@@ -633,18 +697,12 @@ class DataCursor(object):
         position will be set for each event."""
         self._annotation = {}
         self._dot = {}
+
         for ax in self._ax:
-            handles = self._get_data_handles(ax)
-            if len(handles) > 0:
-                x, y = self._get_xy_data(ax, handles[0])
-                xy = x[0], y[0]
-            else:
-                xy = 0, 0
-            self._annotation[ax] = self._new_annotation(ax, xy)
             xl, yl = ax.get_xlim(), ax.get_ylim()
-            self._dot[ax] = self._add_annotation_point(ax, *xy, self.followdot, False)
-            ax.set_xlim(xl)
-            ax.set_ylim(yl)
+            xy = sum(xl) / 2, sum(yl) / 2
+            self._annotation[ax] = self._new_annotation(ax, xy)
+            self._dot[ax] = None
 
     def _snap(self, ax, x, y):
         """Return the plotted value closest to x, y."""
@@ -652,39 +710,41 @@ class DataCursor(object):
         lines = self._get_data_handles(ax)
         nlines = len(self._get_data_handles(ax))
         if nlines == 0:
-            return None, None, None
+            return None
+        best = None
+
         for n, h in enumerate(lines):
-            xdata, ydata = self._get_xy_data(ax, h)
-
-            if isinstance(h, Line2D):
-                xdata_float = ax.convert_xunits(xdata)
-                ydata_float = ax.convert_yunits(ydata)
-            else:
-                xdata_float = xdata
-                ydata_float = ydata
-
-            dx = (xdata_float - x) / np.diff(ax.get_xlim())[0]
-            dy = (ydata_float - y) / np.diff(ax.get_ylim())[0]
-            d = dx ** 2.0 + dy ** 2.0
+            x_pix, y_pix, x_ann, y_ann = self._get_xy_data_display(ax, h)
+            dx = x_pix - x
+            dy = y_pix - y
+            d = dx**2.0 + dy**2.0
             ind = np.nanargmin(d)
             if d[ind] < dmin:
                 dmin = d[ind]
-                best = xdata[ind], ydata[ind], n, ind, h
+                xy_note = x_ann[ind], y_ann[ind]
+                best = n, ind, h
 
-        # x, y, n, ind, handle = self._snap(ax, x, y)
         if best is None:
-            return None, None, None
-        x, y, n, ind, handle = best
+            return None
+        n, ind, handle = best
+        # get original data coordinates
         if self._is3d(ax):
-            # get original data coordinates
-            xo, yo, zo = self._get_xyz(ax, handle, ind)
+            xo, yo, zo = self._get_xyz(handle, ind)
         else:
-            xo, yo = x, y
+            xo, yo = handle.get_xydata()[ind]
             zo = None
         point = SimpleNamespace(
-            x=xo, y=yo, z=zo, ax=ax, n=n, index=ind, handle=handle, nlines=nlines
+            x=xo,
+            y=yo,
+            z=zo,
+            ax=ax,
+            n=n,
+            index=ind,
+            handle=handle,
+            nlines=nlines,
+            xy_note=xy_note,
         )
-        return x, y, point
+        return point
 
     def _fake_getdata(self):
         """
