@@ -10,6 +10,13 @@ from scipy.interpolate import interp1d
 from pyyeti import dsp
 
 
+# temporary patch for numpy < 2.0
+try:
+    np.trapezoid
+except AttributeError:
+    np.trapezoid = np.trapz
+
+
 # FIXME: We need the str/repr formatting used in Numpy < 1.14.
 try:
     np.set_printoptions(legacy="1.13")
@@ -257,7 +264,8 @@ def area(spec):
         This routine is only for specifications with all constant
         db/octave segments. Do not use for general freq vs. psd
         curves, such as output from analysis; use something like
-        :func:`numpy.trapz` instead.
+        :func:`numpy.trapezoid` (or :func:`trapz` for NumPy versions <
+        2.0).
 
     The following derives the equations for computing the area under
     the curve. Each segment is assumed to have a constant db/octave
@@ -335,7 +343,7 @@ def area(spec):
 
     >>> f = np.arange(20, 2000.1, 0.1)
     >>> p = psd.interp(spec, f, linear=False)
-    >>> 3*np.sqrt(np.trapz(p, f, axis=0))   # doctest: +ELLIPSIS
+    >>> 3*np.sqrt(np.trapezoid(p, f, axis=0))   # doctest: +ELLIPSIS
     array([ 18.43...])
     """
     Freq, PSD, _ = proc_psd_spec(spec)
@@ -559,7 +567,8 @@ def rescale(P, F, n_oct=3, freq=None, extendends=True, frange=None):
         >>> from pyyeti import psd
         >>>
         >>> frange = (1.0, np.inf)
-        >>> g = np.random.randn(10000)
+        >>> rng = np.random.default_rng()
+        >>> g = rng.normal(size=10000)
         >>> sr = 400
         >>> f, p = signal.welch(g, sr, nperseg=sr)
         >>> p3, f3, msv3, ms3 = psd.rescale(p, f, frange=frange)
@@ -568,8 +577,8 @@ def rescale(P, F, n_oct=3, freq=None, extendends=True, frange=None):
         >>> p12, f12, msv12, ms12 = psd.rescale(
         ...     p, f, n_oct=12, frange=frange)
         >>>
-        >>> fig = plt.figure('Example')
-        >>> fig.clf()
+        >>> fig = plt.figure('Example', clear=True,
+        ...                  layout='constrained')
         >>> line = plt.semilogx(f, p, label='Linear')
         >>> line = plt.semilogx(f3, p3, label='1/3 Octave')
         >>> line = plt.semilogx(f6, p6, label='1/6 Octave')
@@ -787,7 +796,8 @@ def spl(
 
         >>> import numpy as np
         >>> from pyyeti import psd
-        >>> x = np.random.randn(100000)
+        >>> rng = np.random.default_rng()
+        >>> x = rng.normal(size=100000)
         >>> sr = 4000
         >>> f, spl, oaspl = psd.spl(x, sr, sr, timeslice=len(x)/sr)
         >>> # oaspl should be around 170.75 (since variance = 1):
@@ -827,7 +837,7 @@ def spl(
         pv = (F >= s) & (F <= e)
         F = F[pv]
         P = P[pv]
-    v = P / pref ** 2
+    v = P / pref**2
     return F, 10 * np.log10(v), 10 * np.log10(np.sum(v))
 
 
@@ -841,6 +851,7 @@ def psd2time(
     winends=None,
     gettime=False,
     expand_method="interp",
+    rng=None,
 ):
     r"""
     Generate a 'random' time domain signal given a PSD specification.
@@ -893,6 +904,16 @@ def psd2time(
         constant dB/octave slopes. Use 'rescale' if `spec` provides
         the PSD levels on a center-band scale. See :func:`interp` and
         :func:`rescale` for more information.
+    rng : :class:`numpy.random.Generator` object or None; optional
+        Random number generator. If None, a new generator is created
+        via :func:`numpy.random.default_rng`. Uniform deviates are
+        generated via :func:`rng.random`. Supplying your own `rng` can
+        be handy for parallel applications, for example, when you need
+        repeatability. For illustration, the following creates a
+        PCG-64 DXSM generator and initializes it with a seed of 1::
+
+            from numpy.random import Generator, PCG64DXSM
+            rng = Generator(PCG64DXSM(seed=1))
 
     Returns
     -------
@@ -902,8 +923,9 @@ def psd2time(
     sr : scalar
         The sample rate of the signal (``ppc * fstop``)
     time : 1d ndarray; optional
-        Time vector for the signal starting at zero with step of
-        ``1.0 / sr``: ``time = np.arange(len(sig)) / sr``.
+        Time vector for the signal starting at zero with step of ``1.0
+        / sr``: ``time = np.arange(len(sig)) / sr``. Only returned if
+        `gettime` is True.
 
     Notes
     -----
@@ -990,9 +1012,12 @@ def psd2time(
         >>> spec = np.array([[20,  .0768],
         ...                  [50,  .48],
         ...                  [100, .48]])
-        >>> sig, sr = psd.psd2time(spec, ppc=10, fstart=35,
-        ...                        fstop=70, df=.01,
-        ...                        winends=dict(portion=.01))
+        >>> we = dict(portion=0.01)
+        >>> seed = 1
+        >>> rng = np.random.default_rng(seed)
+        >>> sig, sr = psd.psd2time(
+        ...     spec, 35, 70, ppc=10, df=.01, winends=we, rng=rng,
+        ... )
         >>> sr  # the sample rate should be 70*10 = 700
         700.0
 
@@ -1007,10 +1032,10 @@ def psd2time(
         >>> speci = psd.interp(spec, fi).ravel()
         >>> abs(speci - psdi).max() < .05
         True
-        >>> abs(np.trapz(psdi, fi) - np.trapz(speci, fi)) < .25
+        >>> abs(np.trapezoid(psdi, fi) - np.trapezoid(speci, fi)) < .25
         True
-        >>> fig = plt.figure('Example')
-        >>> fig.clf()
+        >>> fig = plt.figure('Example', clear=True,
+        ...                  layout='constrained')
         >>> a = plt.subplot(211)
         >>> line = plt.plot(np.arange(len(sig))/sr, sig)
         >>> plt.grid(True)
@@ -1083,7 +1108,10 @@ def psd2time(
         amp = np.hstack((np.zeros(ntop), amp))
 
     # generate F(t)
-    phi = np.random.rand(m) * np.pi * 2  # random phase angle
+    # - use uniformly distributed random phase angle:
+    if rng is None:
+        rng = np.random.default_rng()
+    phi = rng.random(m) * np.pi * 2
 
     # force these terms to be pure cosine
     phi[0] = 0.0
@@ -1205,8 +1233,8 @@ def psdmod(sig, sr, nperseg=None, timeslice=1.0, tsoverlap=0.5, getmap=False, **
         ...                     tsoverlap=0.5)
         >>> f3, p3 = psd.psdmod(sig, sr, nperseg=sr)
         >>> spec = spec.T
-        >>> fig = plt.figure('Example')
-        >>> fig.clf()
+        >>> fig = plt.figure('Example', clear=True,
+        ...                  layout='constrained')
         >>> _ = plt.subplot(211)
         >>> _ = plt.plot(t, sig)
         >>> _ = plt.title(r'Input Signal - Specification Level = '
@@ -1223,7 +1251,6 @@ def psdmod(sig, sr, nperseg=None, timeslice=1.0, tsoverlap=0.5, getmap=False, **
         >>> _ = plt.title('PSD')
         >>> _ = plt.xlabel('Frequency (Hz)')
         >>> _ = plt.ylabel(r'PSD ($g^2$/Hz)')
-        >>> _ = plt.tight_layout()
     """
     if nperseg is None:
         nperseg = int(sr / 5)
