@@ -183,7 +183,7 @@ def reldisp(Q, dT, wn):
     C = E * cos(B)
     if wn == 0:
         # See notes above for the derivation of these coefficients:
-        b = np.array([-1.0, -4.0, -1.0]) * dT ** 2 / 6
+        b = np.array([-1.0, -4.0, -1.0]) * dT**2 / 6
     else:
         S = E * sin(B)
         f = dT * wn * wn * wn
@@ -299,7 +299,7 @@ def _negsmeth(resp):
 
 
 def _rmsmeth(resp):
-    return np.sqrt((resp ** 2).mean(axis=0))
+    return np.sqrt((resp**2).mean(axis=0))
 
 
 def fftroll(sig, sr, ppc, frq):
@@ -677,7 +677,9 @@ def srs(
         to use.
     Q : scalar > 0.5
         Dynamic amplification factor :math:`Q = 1/(2\zeta)` where
-        :math:`\zeta` is the fraction of critical damping.
+        :math:`\zeta` is the fraction of critical damping. `Q` must be
+        greater than 0.5 because this routine is for underdamped
+        systems only.
     ic : string; optional
         Specifies how to handle the initial conditions:
 
@@ -964,6 +966,8 @@ def srs(
 
     Raises
     ------
+    ValueError
+        When `Q` <= 0.5; routine is only for underdamped systems.
     ValueError
         When `sr` is None but number of time steps is greater than 1
         OR `ic` is set to 'zero'.
@@ -987,8 +991,7 @@ def srs(
         Compare the upsampling/rolloff methods:
 
         >>> import matplotlib.pyplot as plt
-        >>> _ = plt.figure('Example')
-        >>> plt.clf()
+        >>> _ = plt.figure('Example', clear=True, layout='constrained')
         >>> sr = 200
         >>> t = np.arange(0, 5, 1/sr)
         >>> sig = np.sin(2*np.pi*15*t) + 3*np.sin(2*np.pi*85*t)
@@ -1003,6 +1006,9 @@ def srs(
         >>> _ = plt.title(ttl)
         >>> _ = plt.grid(True)
     """
+    if Q <= 0.5:
+        raise ValueError("Q must be > 0.5 since SRS assumes underdamped equations.")
+
     (coeffunc, methfunc, rollfunc, ptr) = _process_inputs(stype, peak, rolloff, time)
     freq = np.atleast_1d(freq)
     wn = 2 * pi * freq
@@ -1280,6 +1286,11 @@ def vrs(spec, freq, Q, linear, Fn=None, getmiles=False, getresp=False):
            Spectrum - Revision D",
            http://www.vibrationdata.com/tutorials2/vrs.pdf
 
+    Raises
+    ------
+    ValueError
+        When `Q` <= 0.5; routine is only for underdamped systems.
+
     Examples
     --------
     Compute response spectra for example shown in reference. The
@@ -1377,7 +1388,7 @@ def vrs(spec, freq, Q, linear, Fn=None, getmiles=False, getresp=False):
         for i, fn in enumerate(Fn):
             p = freq / fn
             p2z2 = (2 * zeta * p) ** 2
-            t = ((1 + p2z2) / ((1 - p ** 2) ** 2 + p2z2)) * psdfull.T
+            t = ((1 + p2z2) / ((1 - p**2) ** 2 + p2z2)) * psdfull.T
             psd_vrs[i] = t  # npsds x len(freq)
             z_vrs[i] = np.sqrt(np.sum(df * t, axis=1))
         resp = {}
@@ -1390,7 +1401,7 @@ def vrs(spec, freq, Q, linear, Fn=None, getmiles=False, getresp=False):
     for i, fn in enumerate(Fn):
         p = freq / fn
         p2z2 = (2 * zeta * p) ** 2
-        t = ((1 + p2z2) / ((1 - p ** 2) ** 2 + p2z2) * df) * psdfull.T
+        t = ((1 + p2z2) / ((1 - p**2) ** 2 + p2z2) * df) * psdfull.T
         z_vrs[i] = np.sqrt(np.sum(t, axis=1))
 
     if PSD.ndim == 1:
@@ -1800,6 +1811,7 @@ def srs_frf(
         ...     num="Example",
         ...     clear=True,
         ...     figsize=(9, 12),
+        ...     layout='constrained',
         ... )
         >>> _ = ax[0].plot(frf_frq, frf)
         >>>
@@ -1849,7 +1861,6 @@ def srs_frf(
         >>> _ = ax[3].set_xlabel(r"$\Omega$ Frequency (Hz)")
         >>> for axis in ax:
         ...    _ = axis.set_xlim(39.5, 49.5)
-        >>> fig.tight_layout()
 
     In the previous example, we saw that the equivalent sine of the
     input did not give us the original input back, but it was closer
@@ -1880,6 +1891,7 @@ def srs_frf(
         ...     num="Example 2",
         ...     clear=True,
         ...     figsize=(9, 5),
+        ...     layout='constrained',
         ... )
         >>> _ = ax.plot(frf_frq, frf, label="Original Input")
         >>>
@@ -1912,14 +1924,12 @@ def srs_frf(
         >>> _ = ax.set_ylabel("Abs-Acce Eq-Sine (G)")
         >>> _ = ax.set_xlabel(r"$\omega_n$ Frequency (Hz)")
         >>> _ = ax.set_xlim(39.5, 49.5)
-        >>>
-        >>> fig.tight_layout()
     """
     if getresp and scale_by_Q_only:
         raise ValueError("`getresp` and `scale_by_Q_only` cannot both be True")
 
     # compute maximizing Omega / omega_n ratio (see math in docstr):
-    p_peak = Q * np.sqrt(np.sqrt(1 + 2 / Q ** 2) - 1)
+    p_peak = Q * np.sqrt(np.sqrt(1 + 2 / Q**2) - 1)
     frf_frq = np.asarray(frf_frq)
 
     if srs_frq is None:
@@ -1948,7 +1958,7 @@ def srs_frf(
         ws = 2.0 * np.pi * srs_frq
         ms = np.ones(n, float)
         bs = 1 / Q * ws
-        ks = ws ** 2
+        ks = ws**2
 
         # include transfer function peak frequencies in the forcing
         # function (these are close to the natural frequencies):
@@ -1988,7 +1998,7 @@ def srs_frf(
             fw = freqw.reshape(1, -1)
             H = (
                 ks[pvel].reshape(-1, 1)
-                - ms[pvel].reshape(-1, 1) @ fw ** 2
+                - ms[pvel].reshape(-1, 1) @ fw**2
                 + 1j * (bs[pvel].reshape(-1, 1) @ fw)
             )
 
@@ -2003,7 +2013,7 @@ def srs_frf(
             if rb:
                 a[pvrb] = -fs  # / ms ... since ms == 1
             if el:
-                a[pvel] = (fs * freqw ** 2) / H
+                a[pvel] = (fs * freqw**2) / H
             # from relative to absolute acceleration:
             a += fs
             if getresp:
@@ -2101,7 +2111,7 @@ def srsmap(timeslice, tsoverlap, sig, sr, freq, Q, wep=0, **srsargs):
         >>> Q = 20
         >>> mp, t, f = srs.srsmap(2, .5, sig, sr, frq, Q, .02,
         ...                       eqsine=1)
-        >>> _ = plt.figure('Example', clear=True)
+        >>> _ = plt.figure('Example', clear=True, layout='constrained')
         >>> cs = plt.contour(t, f, mp, 40, cmap=cm.plasma)
         >>> # This doesn't work in matplotlib 3.5.0:
         >>> #   cbar = plt.colorbar()
@@ -2112,7 +2122,7 @@ def srsmap(timeslice, tsoverlap, sig, sr, freq, Q, wep=0, **srsargs):
         ...            vmin=cs.cvalues.min(), vmax=cs.cvalues.max()
         ...        )
         >>> sm = plt.cm.ScalarMappable(norm=norm, cmap=cs.cmap)
-        >>> cb = plt.colorbar(sm)  # , ticks=cs.levels)
+        >>> cb = plt.colorbar(sm, ax=plt.gca())  # , ticks=cs.levels)
         >>> #
         >>> _ = plt.xlabel('Time (s)')
         >>> _ = plt.ylabel('Frequency (Hz)')
@@ -2124,7 +2134,8 @@ def srsmap(timeslice, tsoverlap, sig, sr, freq, Q, wep=0, **srsargs):
     .. plot::
         :context: close-figs
 
-        >>> fig = plt.figure("Example 2", clear=True)
+        >>> fig = plt.figure("Example 2", clear=True,
+        ...                  layout='constrained')
         >>> ax = fig.add_subplot(projection="3d")
         >>> x, y = np.meshgrid(t, f)
         >>> surf = ax.plot_surface(x, y, mp, rstride=1, cstride=1,
