@@ -49,7 +49,7 @@ def _check_makeplot(
 
     if makeplot != "no":
         if makeplot == "new" or not plt.get_fignums():
-            plt.figure(figsize=figsize)
+            plt.figure(figsize=figsize, layout="constrained")
 
         if makeplot == "add":
             fig = plt.gcf()
@@ -341,9 +341,9 @@ def fit_circle_3d(basic, makeplot="no"):
          That equation is actually three equations with two of them
          being independent. Therefore, we can solve for ``alpha`` and
          ``beta`` using a least-squares approach (
-         :func:`np.linalg.lstsq`). Then, we can use either of the two
-         equations above to solve for `center`. Note the `center` is
-         in basic coordinates.
+         :func:`numpy.linalg.lstsq`). Then, we can use either of the
+         two equations above to solve for `center`. Note the `center`
+         is in basic coordinates.
       9. The coordinates of all points can now be calculated in the
          local coordinate system (note that the local z-coordinate is
          0.0 for all points)::
@@ -445,7 +445,6 @@ def fit_circle_3d(basic, makeplot="no"):
         ax.plot(*circle_basic, label="Fit")
         axis_equal_3d(ax)
         ax.legend(loc="upper left", bbox_to_anchor=(1.0, 1.0))
-        ax.get_figure().tight_layout()
 
     return circ_parms
 
@@ -697,7 +696,7 @@ def isdiag(A, tol=1e-12):
     --------
     >>> from pyyeti import ytools
     >>> import numpy as np
-    >>> A = np.diag(np.random.randn(5))
+    >>> A = np.diag(np.arange(5.0))
     >>> ytools.isdiag(A)
     True
     >>> A[0, 2] = .01
@@ -715,7 +714,40 @@ def isdiag(A, tol=1e-12):
     return max_off <= tol * max_on
 
 
-def mattype(A, mtype=None):
+def _check_symm_herm(A, mattypes):
+    Atype = 0
+    if np.allclose(A, A.T):
+        Atype |= mattypes["symmetric"]
+
+    elif np.iscomplexobj(A) and np.allclose(A, A.T.conj()):
+        Atype |= mattypes["hermitian"]
+
+    if isdiag(A):
+        Atype |= mattypes["diagonal"] | mattypes["symmetric"]
+        if np.iscomplexobj(A):
+            Atype |= mattypes["hermitian"]
+        d = np.diag(A)
+        if np.allclose(1, d):
+            Atype |= mattypes["identity"]
+
+    return Atype
+
+
+def _check_cholesky(Atype, A, mattypes):
+    chol = None
+    if (Atype & mattypes["symmetric"] and np.isrealobj(A)) or (
+        Atype & mattypes["hermitian"]
+    ):
+        try:
+            chol = linalg.cholesky(A)
+        except linalg.LinAlgError:
+            pass
+        else:
+            Atype |= mattypes["posdef"]
+    return Atype, chol
+
+
+def mattype(A, mtype=None, return_cholesky=False):
     """
     Checks contents of square matrix `A` to see if it is symmetric,
     hermitian, positive-definite, diagonal, and identity.
@@ -731,6 +763,10 @@ def mattype(A, mtype=None):
         this case, True is returned if `A` is of the type specified or
         False otherwise. If None, `Atype` (if `A` is not None) and
         `mattypes` is returned. `mtype` is ignored if `A` is None.
+    return_cholesky : bool; optional
+        If True, the output of :func:`scipy.linalg.cholesky` is
+        returned if computed. Output will be None if `A` is not
+        positive-definite. See example usages below.
 
     Returns
     -------
@@ -753,13 +789,25 @@ def mattype(A, mtype=None):
         Not returned if `mtype` is specified. This is the only return
         if `A` is None.
 
+    chol : 2d ndarray or None
+        See `return_cholesky` above. If returned, `chol` will be the
+        output of :func:`scipy.linalg.cholesky` (with default
+        settings) or None, depending on whether matrix is
+        positive-definite or not.
+
     Notes
     -----
-    Here are some example usages::
+    Here are some example usages:
 
-        mattype(A)               # returns (Atype, mattypes)
-        mattype(A, 'symmetric')  # returns True or False
-        mattype(None)            # returns mattypes
+    ==========================================  =======================
+    Usage                                       Returns
+    ==========================================  =======================
+    mattype(A)                                  (Atype, mattypes)
+    mattype(A, return_cholesky=True)            (Atype, mattypes, chol)
+    mattype(A, 'symmetric')                     True or False
+    mattype(A, 'posdef', return_cholesky=True)  (True or False, chol)
+    mattype(None)                               mattypes
+    ==========================================  =======================
 
     See also
     --------
@@ -809,51 +857,30 @@ def mattype(A, mtype=None):
     if mtype is None:
         if A.ndim != 2 or A.shape[0] != A.shape[1]:
             return Atype, mattypes
-        if np.allclose(A, A.T):
-            Atype |= mattypes["symmetric"]
-            if np.isrealobj(A):
-                try:
-                    linalg.cholesky(A)
-                except linalg.LinAlgError:
-                    pass
-                else:
-                    Atype |= mattypes["posdef"]
-        elif np.iscomplexobj(A) and np.allclose(A, A.T.conj()):
-            Atype |= mattypes["hermitian"]
-            try:
-                linalg.cholesky(A)
-            except linalg.LinAlgError:
-                pass
-            else:
-                Atype |= mattypes["posdef"]
-        if isdiag(A):
-            Atype |= mattypes["diagonal"]
-            d = np.diag(A)
-            if np.allclose(1, d):
-                Atype |= mattypes["identity"]
+
+        Atype = _check_symm_herm(A, mattypes)
+        Atype, chol = _check_cholesky(Atype, A, mattypes)
+
+        if return_cholesky:
+            return Atype, mattypes, chol
         return Atype, mattypes
 
     if A.ndim != 2 or A.shape[0] != A.shape[1]:
         return False
 
     if mtype == "symmetric":
-        return np.allclose(A, A.T)
+        return np.allclose(A, A.T) or isdiag(A)
 
     if mtype == "hermitian":
-        return np.allclose(A, A.T.conj())
+        return np.iscomplexobj(A) and (np.allclose(A, A.T.conj()) or isdiag(A))
 
     if mtype == "posdef":
-        if np.isrealobj(A):
-            if not np.allclose(A, A.T):
-                return False
-        else:
-            if not np.allclose(A, A.T.conj()):
-                return False
-        try:
-            linalg.cholesky(A)
-            return True
-        except linalg.LinAlgError:
-            return False
+        Atype = _check_symm_herm(A, mattypes)
+        Atype, chol = _check_cholesky(Atype, A, mattypes)
+        ret = bool(Atype & mattypes["posdef"])
+        if return_cholesky:
+            return ret, chol
+        return ret
 
     if mtype in ("diagonal", "identity"):
         if isdiag(A):
@@ -954,7 +981,18 @@ def sturm(A, lam):
 
 
 def eig_si(
-    K, M, Xk=None, f=None, p=10, mu=0, tol=1e-6, pmax=None, maxiter=50, verbose=True
+    K,
+    M,
+    *,
+    Xk=None,
+    f=None,
+    p=10,
+    mu=0,
+    tol=1e-6,
+    pmax=None,
+    maxiter=50,
+    verbose=True,
+    rng=None,
 ):
     r"""
     Perform subspace iteration to calculate eigenvalues and eigenvectors.
@@ -962,12 +1000,12 @@ def eig_si(
     Parameters
     ----------
     K : ndarray
-        The stiffness (symmetric).
+        The stiffness (assumed symmetric).
     M : ndarray
-        The mass (positive definite).
+        The mass (assumed positive-definite).
     Xk : ndarray or None
         Initial guess @ eigenvectors; # columns > `p`. If None,
-        random vectors are used from ``np.random.rand()-.5``.
+        random vectors are generated internally; see `rng` below.
     f : scalar or None
         Desired cutoff frequency in Hz. `pmax` will override this if
         set. Takes precedence over `p` if both are input.
@@ -985,15 +1023,25 @@ def eig_si(
         Maximum number of iterations.
     verbose : bool
         If True, print status message for each iteration.
+    rng : :class:`numpy.random.Generator` object or None; optional
+        Random number generator. If None, a new generator is created
+        via :func:`numpy.random.default_rng`. Uniform deviates are
+        generated via :func:`rng.random`. Supplying your own `rng` can
+        be handy for parallel applications, for example, when you need
+        repeatability. For illustration, the following creates a
+        PCG-64 DXSM generator and initializes it with a seed of 1::
+
+            from numpy.random import Generator, PCG64DXSM
+            rng = Generator(PCG64DXSM(seed=1))
 
     Returns
     -------
     lam : ndarray
         Ideally, `p` converged eigenvalues.
     phi : ndarray
-        Ideally, p converged eigenvectors.
+        Ideally, `p` converged eigenvectors.
     phiv : ndarray
-        First p columns are `phi`, others are leftover iteration
+        First `p` columns are `phi`, others are leftover iteration
         vectors which may be a good starting point for a second call.
 
     Notes
@@ -1030,6 +1078,14 @@ def eig_si(
     eigenvectors (such as the output `phiv` may be), letting `Xk`
     start as random seems to work well.
 
+    Routine follows the basic algorithm as outlined in [#ss1]_.
+
+    References
+    ----------
+    .. [#ss1] Bathe KJ, Ramaswamy S; "An Accelerated Subspace Iteration
+           Method", Journal of Computer Methods in Applied Mechanics
+           and Engineering, Vol 23, 1980, pp 313–331.
+
     Examples
     --------
     >>> from pyyeti import ytools
@@ -1043,16 +1099,19 @@ def eig_si(
     >>> print(abs(w))
     [  0.   5.  15.]
     >>> import scipy.linalg as linalg
-    >>> k = np.random.randn(40, 40)
-    >>> m = np.random.randn(40, 40)
+    >>> rng = np.random.default_rng()
+    >>> k = rng.normal(size=(40, 40))
+    >>> m = rng.normal(size=(40, 40))
     >>> k = np.dot(k.T, k) * 1000
     >>> m = np.dot(m.T, m) * 10
     >>> w1, phi1 = linalg.eigh(k, m, subset_by_index=(0, 14))
-    >>> w2, phi2, phiv2 = ytools.eig_si(k, m, p=15, mu=-1, tol=1e-12,
-    ...                                 verbose=False)
+    >>> w2, phi2, phiv2 = ytools.eig_si(
+    ...     k, m, p=15, mu=-1, tol=1e-12, verbose=False
+    ... )
     >>> fcut = np.sqrt(w2.max())/2/np.pi * 1.001
-    >>> w3, phi3, phiv3 = ytools.eig_si(k, m, f=fcut, tol=1e-12,
-    ...                                 verbose=False)
+    >>> w3, phi3, phiv3 = ytools.eig_si(
+    ...     k, m, f=fcut, tol=1e-12, verbose=False
+    ... )
     >>> print(np.allclose(w1, w2))
     True
     >>> print(np.allclose(np.abs(phi1), np.abs(phi2)))
@@ -1063,6 +1122,8 @@ def eig_si(
     True
     """
     n = np.size(K, 0)
+    # K = (K + K.T) / 2
+    # M = (M + M.T) / 2
     if f is not None:
         # use sturm sequence check to determine p:
         lamk = (2 * np.pi * f) ** 2
@@ -1086,45 +1147,54 @@ def eig_si(
     else:
         c = 0
     if c < q:
+        if rng is None:
+            rng = np.random.default_rng()
+        deviates = rng.random((n, q - c)) - 0.5
         if Xk is None:
-            Xk = np.random.rand(n, q) - 0.5
+            Xk = deviates
         else:
-            Xk = np.hstack((Xk, np.random.rand(n, q - c) - 0.5))
+            Xk = np.hstack((Xk, deviates))
     elif c > q:
         Xk = Xk[:, :q]
+
     lamk = np.ones(q)
     nconv = 0
     loops = 0
     tolc = 1
-    posdef = mattype(None)["posdef"]
     eps = np.finfo(float).eps
-    while (tolc > tol or nconv < p) and loops < maxiter:
+    # while (tolc > tol or nconv < p) and loops < maxiter:
+    while nconv < p and loops < maxiter:
         loops += 1
         lamo = lamk
-        MXk = np.dot(M, Xk)
-        Xkbar = linalg.lu_solve(Kd, MXk)
-        Mk = np.dot(np.dot(Xkbar.T, M), Xkbar)
-        Kk = np.dot(Xkbar.T, MXk)
+        MXk = M @ Xk
+        Xkbar = linalg.lu_solve(Kd, MXk)  # eq 5
+        Kk = Xkbar.T @ MXk  # eq 6
+        Mk = Xkbar.T @ M @ Xkbar  # eq 7
+
+        # Kk = (Kk + Kk.T) / 2
+        # Mk = (Mk + Mk.T) / 2
 
         # solve subspace eigenvalue problem:
-        mtp = mattype(Mk)[0]
-        if not mtp & posdef:
+        mtp, Mkuu = mattype(Mk, "posdef", return_cholesky=True)
+        if not mtp:
             factor = 1000 * eps
             pc = 0
             while 1:
                 pc += 1
                 Mk += np.diag(np.diag(Mk) * factor)
                 factor *= 10.0
-                mtp = mattype(Mk)[0]
-                if mtp & posdef or pc > 5:
+                mtp, Mkuu = mattype(Mk, "posdef", return_cholesky=True)
+                if mtp or pc > 5:
                     break
 
-        if mtp & posdef:
-            Mkll = linalg.cholesky(Mk, lower=True)
-            Kkmod = linalg.solve(Mkll, linalg.solve(Mkll, Kk).T).T
+        if mtp:
+            Mkll = Mkuu.T  # linalg.cholesky(Mk, lower=False).T
+            Kkmod = linalg.solve_triangular(
+                Mkll, linalg.solve_triangular(Mkll, Kk, lower=True).T, lower=True
+            )
             Kkmod = (Kkmod + Kkmod.T) / 2
-            lamk, Qmod = linalg.eigh(Kkmod)
-            Q = linalg.solve(Mkll.T, Qmod)
+            lamk, Qmod = linalg.eigh(Kkmod)  # eq 8
+            Q = linalg.solve_triangular(Mkll, Qmod, lower=True, trans="T")
         else:
             raise ValueError(
                 "subspace iteration failed, reduced mass"
@@ -1146,7 +1216,7 @@ def eig_si(
             if verbose:
                 print("Iteration 1 completed")
             nconv = 0
-        Xk = np.dot(Xkbar, Q)
+        Xk = Xkbar @ Q
     return lamk[:p] + mu, Xk[:, :p], Xk
 
 
@@ -1194,8 +1264,7 @@ def gensweep(ppc, fstart, fstop, rate):
         >>> from pyyeti import ytools
         >>> import matplotlib.pyplot as plt
         >>> sig, t, f = ytools.gensweep(10, 1, 12, 8)
-        >>> _ = plt.figure('Example')
-        >>> plt.clf()
+        >>> _ = plt.figure('Example', clear=True, layout='constrained')
         >>> _ = plt.subplot(211)
         >>> _ = plt.plot(t, sig)
         >>> _ = plt.title('Sine Sweep vs Time')
@@ -1206,7 +1275,6 @@ def gensweep(ppc, fstart, fstop, rate):
         >>> _ = plt.title('Sine Sweep vs Frequency')
         >>> _ = plt.xlabel('Frequency (Hz)')
         >>> _ = plt.xlim([f[0], f[-1]])
-        >>> _ = plt.tight_layout()
     """
     # make a unity sine sweep
     rate = rate / 60.0
@@ -1329,9 +1397,9 @@ def reorder_dict(ordered_dict, keys, where):
     ...                    ('two', 2),
     ...                    ('three', 3)))
     >>> dct
-    OrderedDict([('one', 1), ('two', 2), ('three', 3)])
+    OrderedDict({'one': 1, 'two': 2, 'three': 3})
     >>> reorder_dict(dct, ['three', 'two'], 'first')
-    OrderedDict([('three', 3), ('two', 2), ('one', 1)])
+    OrderedDict({'three': 3, 'two': 2, 'one': 1})
     """
 
     def reorder_keys(all_keys, keys, where):
@@ -1479,7 +1547,7 @@ def compmat(a, b, filterval=0.0, method="abs", pdiff_tol=0, verbose=5):
         Used to filter out small numbers; the exact usage depends on
         `method`. In all cases however, both the `a` and `b` values
         (absolute value wise) have to be below the filter value for
-        the comparison to be ignored.
+        the comparison to be ignored. Must be >= 0.0.
     method : string; optional
         Specifies how to use filter:
 
@@ -1525,6 +1593,11 @@ def compmat(a, b, filterval=0.0, method="abs", pdiff_tol=0, verbose=5):
     ------
     ValueError
         If `a` and `b` are different sizes
+    ValueError
+        If number of dimensions of `a` and `b` > 2
+    ValueError
+        If `filterval` < 0.0
+
 
     Notes
     -----
@@ -1602,6 +1675,12 @@ def compmat(a, b, filterval=0.0, method="abs", pdiff_tol=0, verbose=5):
             f"matrix sizes do not match: a.shape={a.shape}, b.shape={b.shape}"
         )
 
+    if a.ndim > 2:
+        raise ValueError(f"arrays have {a.ndim} dimensions; must be <= 2")
+
+    if filterval < 0.0:
+        raise ValueError(f"`filterval` is {filterval}, but must be >= 0.0")
+
     if np.iscomplexobj(a) ^ np.iscomplexobj(b):
         # if only one is complex:
         if np.isrealobj(a):
@@ -1760,3 +1839,70 @@ def max_complex_vector_sum(x, y):
         s, c = s[0], c[0]
     theta = np.arctan2(s, c)
     return (c * x + s * y), theta, c, s
+
+
+# Only define numba_interp() if Numba is installed.
+try:
+    import numba
+except ImportError:
+    pass
+else:
+
+    def numba_interp(x: np.ndarray, xp: np.ndarray, fp: np.ndarray) -> np.ndarray:
+        """
+        Apply np.interp to each row of 2d array.
+
+        Uses Numba to parallelize interpolation across rows of `fp`.
+
+        Parameters
+        ----------
+        x : 1d ndarray
+            The x-coordinates at which to evaluate the interpolated
+            values.
+        xp : 1d ndarray
+            The x-coordinates of the data points. 1-dimensional.
+        fp : 2d ndarray
+            The y-coordinates of the data points, with shape [Any #
+            rows, xp.shape[0]]
+
+        Returns
+        -------
+        2d ndarray
+            The interpolated values, with shape [fp.shape[0],
+            x.shape[0]]
+        """
+
+        # Check dimensions of inputs
+        assert x.ndim == 1, f'"x" must be 1-dimensional. ndim = {x.ndim}'
+        assert xp.ndim == 1, f'"xp" must be 1-dimensional. ndim = {xp.ndim}'
+        assert fp.ndim == 2, f'"fp" must be 2-dimensional. ndim = {fp.ndim}'
+
+        # Ensure that shapes of xp and fp agree
+        assert fp.shape[1] == xp.shape[0], f'fp.shape[1] must match xp.shape[0]. {fp.shape[1]} != {xp.shape[0]}'
+
+        # Ensure that x and xp are float64
+        x = x.astype(np.float64, copy=False)
+        xp = xp.astype(np.float64, copy=False)
+
+        # Depending on the data type of fp, call the Numba-compiled Numpy interp,
+        # or coerce to float64 and call it.
+        if fp.dtype in ['float32', 'float64', 'complex64', 'complex128']:
+            f = _numba_interp(x, xp, fp)
+        else:
+            f = _numba_interp(x, xp, fp.astype(np.float64, copy=False))
+
+        return f
+
+    @numba.njit(
+        parallel=True,
+        nogil=True,
+        cache=True,
+    )
+    def _numba_interp(x, xp, fp):  # pragma: no cover
+        n_rows = fp.shape[0]
+        n_cols = x.shape[0]
+        f = np.zeros((n_rows, n_cols), dtype=fp.dtype)
+        for i_row in numba.prange(n_rows):
+            if fp[i_row, :].any():
+                f[i_row, :] = np.interp(x, xp, fp[i_row, :])
+        return f
